from typing import Iterator, Set

from pydantic.class_validators import root_validator

from structurizr2csv.coordinates import Dimensions, Position
from structurizr2csv.elements.base_box import BaseBox
from structurizr2csv.enums import Terminology
from structurizr2csv.settings import FILL_COLORS, STROKE_COLORS


class LegendRow(BaseBox):

    box_type: str
    _height: int = 30

    @root_validator(pre=True)
    def init_base_box_attrs(cls, values):  # @NoSelf
        """Initialize `position` and `dimensions`"""
        values.update({"dimensions": Dimensions(x=180, y=cls._height)})
        return values

    @property
    def style(self) -> str:
        return "legend_row"

    @property
    def label(self) -> str:
        match self.box_type.split("-"):
            case c4_name, tag:
                return "{} {}".format(
                    tag.title(), getattr(Terminology, c4_name).value  # noqa
                )
            case [c4_name]:  # noqa
                return getattr(Terminology, c4_name).value  # noqa
            case _:
                raise NotImplementedError

    @property
    def csv_data(self):
        data = super().csv_data
        data.update(
            {
                "c4Name": self.label,
                "id": f"id_{self.box_type}",
                "parent": Legend._id,
                "styleKey": self.style,
                "labelKey": self.style,
                "left": self.position.x,
                "top": self.position.y,
                "width": self.dimensions.x,
                "height": self.dimensions.y,
                "fill": FILL_COLORS.get(self.box_type, ""),
                "stroke": STROKE_COLORS.get(self.box_type, ""),
            }
        )
        return data


class LegendHeader(LegendRow):
    box_type: str = "legend"

    @property
    def style(self) -> str:
        return "legend_header"


class Legend(BaseBox):
    box_types: Set[str]
    _id: str = "id_legend_container"

    @root_validator(pre=True)
    def init_base_box_attrs(cls, values):  # @NoSelf
        """Initialize `dimensions`"""
        values.update(
            {
                "dimensions": Dimensions(
                    x=180, y=LegendRow._height * (1 + len(values["box_types"]))
                )
            }
        )
        return values

    @property
    def csv_data(self):
        data = super().csv_data
        data.update(
            {
                "id": self._id,
                "styleKey": '"strokeColor=none;"',  # don't display the border
                "left": self.position.x,
                "top": self.position.y,
                "width": self.dimensions.x,
                "height": self.dimensions.y,
            }  # pyright: reportGeneralTypeIssues=false
        )
        return data

    def to_csv(self) -> Iterator[str]:
        yield super().to_csv()
        yield LegendHeader(position=Position(x=0, y=0)).to_csv()
        for index, box_type in enumerate(
            sorted(
                # sort by color first, then by label
                self.box_types,
                key=lambda box_type: (FILL_COLORS[box_type], box_type),
            ),
            1,
        ):
            yield LegendRow(
                position=Position(x=0, y=index * LegendRow._height), box_type=box_type
            ).to_csv()



from functools import cached_property
from typing import Dict, List

from pydantic.class_validators import root_validator

from structurizr2csv.coordinates import Dimensions, Position
from structurizr2csv.elements.base_box import BaseBox
from structurizr2csv.enums import Tag, Terminology
from structurizr2csv.settings import FILL_COLORS, POSITION_FACTOR, STROKE_COLORS
from structurizr2csv.workspace import Component, Container, ElementView
from structurizr2csv.workspace_extension import Element


class Box(BaseBox):
    """Convert a basic element in structurizr to a box in diagrams.net"""

    class Config:
        keep_untouched = (cached_property,)

    element: ElementView

    @root_validator(pre=True)
    def init_base_box_attrs(cls, values):  # type: ignore
        """Initialize `position` and `dimensions` based on `element`"""
        element = values["element"]
        values["position"] = Position(
            x=element.x * POSITION_FACTOR,
            y=element.y * POSITION_FACTOR,
        )
        values["dimensions"] = Dimensions(x=cls.width(element), y=cls.height(element))
        return values

    @classmethod
    def width(cls, element):
        """Box's width in diagrams.net"""
        match element.c4_type:
            case Terminology.person:
                return 200
            case _:
                return 240

        raise NotImplementedError

    @classmethod
    def height(cls, element):
        """Box's height in diagrams.net"""
        match element.c4_type:
            case Terminology.person:
                return 180
            case _:
                return 120

        raise NotImplementedError

    @property
    def style(self) -> str:
        """Returns the style in diagrams.net"""
        style = self.definition.c4_type.name  # type: ignore
        for tag in (
            Tag.HEXAGON,
            Tag.CYLINDER,
            Tag.PIPE,
            Tag.WEB_BROWSER,
            Tag.DATABASE,
            Tag.MOBILE_APP,
        ):
            if tag.value in self.tags:
                match tag:  # handle aliases
                    case Tag.DATABASE:
                        tag = Tag.CYLINDER
                    case Tag.MOBILE_APP:
                        tag = Tag.WEB_BROWSER
                return tag.value.lower().replace("_", "-").replace(" ", "-")

        return style

    @property
    def box_type(self) -> str:
        """Returns the shape's type"""
        c4_type = self.definition.c4_type  # pyright: reportGeneralTypeIssues=false
        if Tag.EXISTING_SYSTEM.value in self.tags:
            return f"{c4_type.name}-existing"
        elif self.is_internal or c4_type == Terminology.person:
            return c4_type.name
        else:
            return f"{c4_type.name}-external"

    def get_color(self, colors: Dict[str, str]) -> str:
        return colors[self.box_type]

    @property
    def definition(self) -> Element:
        """Returns the model element for this view element"""
        return self.element.definition  # type: ignore

    @property
    def is_internal(self) -> bool:
        """Tells if the box is internal to the company or not"""
        return (
            isinstance(self.definition, (Component, Container))
            or self.definition.is_internal
        )

    @property
    def tags(self) -> List[str]:
        return self.definition.tags.split(",") if self.definition.tags else []

    @property
    def c4_type(self) -> Terminology:
        return self.definition.c4_type

    @property
    def csv_data(self) -> Dict[str, str]:
        type_ = self.c4_type.value
        if getattr(self.definition, "technology", None):
            type_ += f": {self.definition.technology}"

        return {
            "c4Name": self.definition.name,
            "c4Type": type_,
            "c4Description": self.definition.safe_description,
            "id": self.element.id,
            "parent": self.parent_id,
            "styleKey": self.style,
            "labelKey": "box",
            "left": self.relative_position.x,
            "top": self.relative_position.y,
            "width": self.dimensions.x,
            "height": self.dimensions.y,
            "fill": self.get_color(FILL_COLORS),
            "stroke": self.get_color(STROKE_COLORS),
        }



from typing import Dict, Iterable, Optional, Union

from pydantic import BaseModel

from structurizr2csv.enums import Terminology
from structurizr2csv.utils import camel_case
from structurizr2csv.workspace import (
    Component,
    ComponentView,
    Container,
    ContainerView,
    DeploymentView,
    DynamicView,
    ElementView,
    Location,
    Model,
    Person,
    RelationshipView,
    SoftwareSystem,
    SystemContextView,
    SystemLandscapeView,
    Views,
    Workspace,
)

View = Union[
    SystemLandscapeView,
    SystemContextView,
    ContainerView,
    ComponentView,
    DynamicView,
    DeploymentView,
]
Element = Union[Person, SoftwareSystem, Container, Component]


class WorkspaceWrapper:
    """Extends the structurizr's workspace parser to add convenient features.

    The file `workspace.py` is automatically generated by `datamodel-codegen`.
    To ease further updates, we don't want to modify that file by hand, so we
    use this class to dynamically add some attributes and methods.
    """

    def __init__(self, workspace: Workspace):

        self._workspace = workspace

        if self.model:
            # this is how you dynamically add attributes to a pydantic model
            object.__setattr__(
                self.model, "object_ids", self.retrieve_model_object_ids()
            )

            for view in self.iterate_views():
                for element in view.elements or []:
                    self.extend_element(element)
                for relationship in view.relationships or []:
                    self.extend_element(relationship)

    @property
    def model(self) -> Optional[Model]:
        return self._workspace.model

    @property
    def views(self) -> Optional[Views]:
        return self._workspace.views

    def extend_element(self, elt: Union[ElementView, RelationshipView]):
        """Do a lot of dirty stuff to add attributes to pydantic model.

        AFAIK there's no cleaner way to extend a pydantic model that has already been
        defined. Subclassing etc. doesn't work as with other "normal" classes.
        """
        assert self.model, "Workspace's model is empty - no elements to extend"  # nosec

        definition = self.model.object_ids[elt.id]  # type: ignore

        # this is how you dynamically add attributes to a pydantic model...
        object.__setattr__(
            definition,
            "c4_type",
            Terminology[camel_case(definition.__class__.__name__)],
        )
        object.__setattr__(
            definition,
            "safe_description",
            (
                '"{}"'.format(definition.description.replace('"', '\\"'))
                if definition.description
                else ""
            ),
        )

        if isinstance(definition, (Person, SoftwareSystem)):
            object.__setattr__(
                definition,
                "is_internal",
                # compare with values, because the auto-generated workspace might
                # define other enums called "Location1", "Location2"...
                definition.location.value == Location.Internal.value
                if definition.location
                else False,
            )

        object.__setattr__(elt, "definition", definition)
        object.__setattr__(
            elt, "c4_type", definition.c4_type  # pyright: reportGeneralTypeIssues=false
        )

    def retrieve_model_object_ids(self) -> Dict:
        """Recursively retrieve all model objects' IDs."""
        ids = {}

        def map_objects(container: BaseModel, attrname: str):
            for obj in getattr(container, attrname, []) or []:

                ids.setdefault(obj.id, obj)

                map_objects(obj, "relationships")
                map_objects(obj, "containers")
                map_objects(obj, "components")

        map_objects(self.model, "people")
        map_objects(self.model, "softwareSystems")

        return ids

    def iterate_views(self) -> Iterable[View]:
        """Yield all views for this workspace."""
        for view_type in (
            attr
            for attr in self.views.__dict__.keys()
            if (attr.endswith("Views") or attr == "Persons")
        ):
            views = getattr(self.views, view_type)
            if views:
                for view in getattr(self.views, view_type):
                    yield view



#!python3
# avoid absolute path in shebang, so that it also works on windows

import argparse
import glob
from pathlib import Path

from structurizr2csv.dsl2json import DSL2JSON
from structurizr2csv.elements.diagram import Diagram
from structurizr2csv.utils import Directory
from structurizr2csv.workspace import Workspace
from structurizr2csv.workspace_extension import WorkspaceWrapper


class Converter:
    @classmethod
    def parse_args(cls) -> argparse.Namespace:
        parser = argparse.ArgumentParser(
            description=(
                "Convert structurizr DSL or JSON (for describing C4 models) to "
                "diagrams.net CSV format"
            )
        )
        parser.add_argument(
            "input_path",
            type=Path,
            help=(
                "input workspace file, either *.dsl or *.json. "
                "If you pass a *.json file, it is expected to define the positions "
                "of each elements. If you pass a *.dsl file, it will be "
                "automatically transformed to JSON and GraphViz auto layout will "
                "be applied (this is done with the official Docker image for "
                "structurizr/cli, so you need a running Docker system for this to "
                "work with a *.dsl file). When using the '-a {dsl,json}' option, "
                "this must be the path to a directory."
            ),
        )
        parser.add_argument(
            "-a",
            "--all",
            type=str,
            choices=["dsl", "json"],
            dest="process_all",
            help="process all the files in 'input_path' that are of the given type",
        )
        parser.add_argument(
            "-o",
            "--output",
            type=Path,
            dest="output_path",
            help=(
                "output base directory for the CSV files "
                "(workspace basename will be appended)"
            ),
            default="output",
        )
        return parser.parse_args()

    @classmethod
    def get_workspace(cls, filepath: Path) -> Workspace:
        with open(filepath, "r") as fp:
            return Workspace.parse_raw(fp.read())

    @classmethod
    def ensure_directory_exists(cls, directory: Path):
        Path(directory).mkdir(parents=True, exist_ok=True)

    def process_views(self, workspace: Workspace, output_path: Path):
        Directory.create_or_clean(output_path)

        for view in WorkspaceWrapper(workspace).iterate_views():
            Diagram(view=view, model=workspace.model, output_path=output_path).process()

    def process_workspace(self, input_path: Path, output_path: Path):
        workspace_suffix = input_path.suffix.lower()
        if workspace_suffix not in (".dsl", ".json"):
            raise RuntimeError("The workspace file suffix must be either .dsl or .json")

        if workspace_suffix == ".dsl":
            input_path = DSL2JSON(input_path).convert()

        self.ensure_directory_exists(output_path)
        workspace = self.get_workspace(input_path)

        self.process_views(workspace, output_path / input_path.stem)

    def run(self):
        args = self.parse_args()

        if not args.input_path.exists():
            raise FileNotFoundError(f"{args.input_path} does not exist")

        if args.process_all:
            for filename in glob.glob(f"{args.input_path}/*.{args.process_all}"):
                print(f"Converting {filename}")
                self.process_workspace(Path(filename), args.output_path)
        else:
            self.process_workspace(args.input_path, args.output_path)


def main():
    Converter().run()


if __name__ == "__main__":
    main()



from collections import OrderedDict
from functools import cached_property
from itertools import groupby
from operator import attrgetter
from pathlib import Path
from typing import Dict, Iterator, List, Optional, Tuple

from pydantic import BaseModel, Field

from structurizr2csv.coordinates import Position
from structurizr2csv.elements.base_box import BaseBox
from structurizr2csv.elements.box import Box
from structurizr2csv.elements.connector import Connector
from structurizr2csv.elements.group import Group
from structurizr2csv.elements.legend import Legend
from structurizr2csv.elements.title import Title
from structurizr2csv.enums import Terminology
from structurizr2csv.settings import LEGEND_OFFSET_FROM_TITLE
from structurizr2csv.style import Style
from structurizr2csv.workspace import Component, Container, Model, Person
from structurizr2csv.workspace_extension import View


class Diagram(BaseModel):
    """Convert a view in structurizr to a diagram in diagrams.net"""

    class Config:
        keep_untouched = (cached_property,)

    view: View
    model: Model
    output_path: Path

    # Map relationship's description to diagram.net connectors, so we can
    # create a single object for all connectors sharing the same description
    connectors: Dict[Tuple[str], Connector] = Field(default_factory=dict)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # I miss a better solution... c.f. structurizr2csv.utils.auto_index
        try:
            Connector.reset_counter()
        except AttributeError:  # Connector has never been instantiated yet
            pass
        try:
            Group.reset_counter()
        except AttributeError:  # Group has never been instantiated yet
            pass

    @property
    def connector_columns(self) -> str:
        """Return the list of comma-separated connectors' IDs"""
        return ",".join(connector.id for connector in self.connectors.values())

    @cached_property
    def boxes(self) -> List[Box]:
        """Return all boxes for this structurizr view"""
        return [
            Box(element=element)
            for element in sorted(
                self.view.elements,
                key=lambda element: int(
                    element.id
                ),  # pyright: reportGeneralTypeIssues=false
            )
        ]

    @cached_property
    def primary_children(self) -> List[Box]:
        """Return all children of the primary group for this view.

        Depending on the view's scope, this could be all boxes that are internal
        to the company, or all containers of a given software system...
        """
        key = None
        match self.view.__class__.__name__:
            case "SystemLandscapeView" | "SystemContextView":
                if self.view.enterpriseBoundaryVisible:
                    key = attrgetter("is_internal")
            case "ContainerView":
                key = lambda box: isinstance(box.definition, Container)  # noqa: E731
            case "ComponentView":
                key = lambda box: isinstance(box.definition, Component)  # noqa: E731

        return list(filter(key, self.boxes)) if key else []

    @cached_property
    def primary_group(self) -> Optional[Group]:
        """Return the primary group for this view.

        Depending on the view's scope, this could be the company,
        or a given software system, or a given container...
        """
        match self.view.__class__.__name__:
            case "SystemLandscapeView" | "SystemContextView":
                if self.view.enterpriseBoundaryVisible:
                    try:
                        name = self.model.enterprise.name  # type:ignore
                    except AttributeError:
                        assert self.model.enterprise is None
                        return None
                    else:
                        c4_type = Terminology.enterprise
                else:
                    return None
            case "ContainerView":
                name = self.model.object_ids[self.view.softwareSystemId].name
                c4_type = Terminology.softwareSystem
            case "ComponentView":
                name = self.model.object_ids[self.view.containerId].name
                c4_type = Terminology.container
            case _:
                return None

        children = self.primary_children + self.secondary_groups
        group = Group.compute_from_boxes(children, name=name, c4_type=c4_type)
        for child in children:
            if child.parent is None:  # leave the boxes that are in secondary groups
                child.parent = group
        return group

    @cached_property
    def secondary_groups(self) -> List[Group]:
        """Returns the secondary groups.

        Those are necessarily user defined groups that have no particular
        meaning to structurizr.
        """
        groups = []
        for name, boxes in groupby(self.boxes, attrgetter("definition.group")):
            children = [
                box
                for box in boxes
                if box in self.primary_children or isinstance(box.definition, Person)
            ]
            if children and name:
                group = Group.compute_from_boxes(
                    children,
                    name=name,
                )
                groups.append(group)
                for box in children:
                    box.parent = group
        return groups

    def connectors_to_csv(self) -> Iterator[str]:
        """Yield the CSV rows for all connectors."""
        for relationship in self.view.relationships or []:
            definition = relationship.definition
            keys = OrderedDict(
                (
                    ("description", definition.description),
                    ("technology", definition.technology),
                )
            )

            try:
                connector = self.connectors[keys.values()]
            except KeyError:
                connector = self.connectors.setdefault(keys.values, Connector(**keys))
                yield str(connector)  # str representation for diagrams.net CSV

            connector.instances[definition.destinationId].add(definition.sourceId)

    def boxes_to_csv(self) -> Iterator[str]:
        """Yield the CSV rows for all boxes."""
        for box in self.boxes:
            yield box.to_csv()
            for connector in self.connectors.values():  # append connectors data
                yield ","
                if box.element.id in connector.instances.keys():
                    yield (
                        '"{}"'.format(
                            ",".join(sorted(connector.instances[box.element.id]))
                        )
                    )
            yield "\n"

    def make_title(self) -> Title:
        """Create the diagram's title instance."""
        title = self.view.title
        if not title:
            match self.view.__class__.__name__:
                case "SystemLandscapeView":
                    try:
                        title = (
                            self.model.enterprise.name
                        )  # pyright: reportOptionalMemberAccess=false
                    except AttributeError:
                        assert self.model.enterprise is None
                        title = ""
                case "SystemContextView" | "ContainerView":
                    title = self.model.object_ids[self.view.softwareSystemId].name
                case "ComponentView":
                    title = self.model.object_ids[self.view.containerId].name
        return Title(view=self.view, title=title or "")

    def write_headers(self, fp):
        """Write diagrams.net CSV headers.

        This is more or less where we tell diagrams.net which CSV columns exist,
        and if they are used to hold meta data, editable data or connections.
        """
        data_columns = ",".join(BaseBox._data_columns)
        meta_columns = ",".join(BaseBox._meta_columns)

        columns = f"{meta_columns},{self.connector_columns}"

        fp.write(f"# ignore: {columns}\n")
        fp.write(f"{data_columns},{columns}\n")

    def write_group(self, fp, group: Group):
        """Write a group to CSV format."""
        fp.write("{}{}\n".format(group.to_csv(), "," * len(self.connectors)))

    def write_title(self, title: Title, fp):
        """Write the diagram's title to CSV format."""
        fp.write("{}{}\n".format(title.to_csv(), "," * len(self.connectors)))

    def write_legend(self, title: Title, fp):
        """Write the diagram's legend to CSV format."""
        for row in Legend(
            view=self.view,
            position=Position(
                x=title.position.x,
                y=title.position.y + title.dimensions.y + LEGEND_OFFSET_FROM_TITLE,
            ),
            box_types=set(box.box_type for box in self.boxes),
        ).to_csv():
            fp.write("{}{}\n".format(row, "," * len(self.connectors)))

    def process(self):
        """Export the structurizr view to diagrams.net CSV format."""
        styles = Style.build_headers()

        output_filepath = self.output_path / f"{self.view.key}.csv"

        with open(output_filepath, "w") as fp:
            fp.write(f"## {self.view.key}\n{styles}\n")

            for output in self.connectors_to_csv():
                fp.write(output)

            self.write_headers(fp)

            if self.primary_group:
                self.write_group(fp, self.primary_group)

            for group in self.secondary_groups:
                group.parent = self.primary_group
                self.write_group(fp, group)

            for output in self.boxes_to_csv():
                fp.write(output)

            title = self.make_title()
            self.write_title(title, fp)
            self.write_legend(title, fp)

        print(f"Created/updated {output_filepath}")



from datetime import datetime

from pydantic.class_validators import root_validator

from structurizr2csv.coordinates import Dimensions, Position
from structurizr2csv.elements.base_box import BaseBox
from structurizr2csv.enums import Terminology
from structurizr2csv.settings import POSITION_FACTOR
from structurizr2csv.utils import camel_case
from structurizr2csv.workspace_extension import View


class Title(BaseBox):

    view: View
    title: str

    @root_validator(pre=True)
    def init_base_box_attrs(cls, values):  # @NoSelf
        """Initialize `position` and `dimensions`"""
        values.update(
            {
                "position": Position(
                    x=0,
                    y=values["view"].dimensions.height * POSITION_FACTOR
                    if values["view"].dimensions
                    else 0,
                ),
                "dimensions": Dimensions(x=600, y=40),
            }
        )
        return values

    @property
    def key(self) -> str:
        return self.view.key

    @property
    def description(self) -> str:
        return '"{}"'.format(datetime.today().strftime("%A %d. %B %Y, %H:%M:%S"))

    @property
    def style(self) -> str:
        return "title"

    @property
    def name(self) -> str:
        classname = self.view.__class__.__name__
        diagram_type = getattr(
            Terminology, camel_case(classname).replace("View", "")
        ).value
        return f"[{diagram_type}] {self.title}"

    @property
    def csv_data(self):
        data = super().csv_data
        data.update(
            {
                "c4Name": self.name,
                "c4Description": self.description,
                "id": "title",
                "styleKey": self.style,
                "labelKey": self.style,
                "left": self.position.x,
                "top": self.position.y,
                "width": self.dimensions.x,
                "height": self.dimensions.y,
            }  # pyright: reportGeneralTypeIssues=false
        )
        return data



from typing import TYPE_CHECKING, List, Optional

from structurizr2csv.coordinates import Dimensions, Position
from structurizr2csv.elements.base_box import BaseBox

if TYPE_CHECKING:
    from .box import Box  # @UnusedImport


class BoundingBox(BaseBox):
    @classmethod
    def compute_from_boxes(
        cls, boxes: List["Box"], **kwargs
    ) -> Optional["BoundingBox"]:
        if not boxes:
            return None

        position = Position(
            x=min(box.position.x for box in boxes),
            y=min(box.position.y for box in boxes),
        )
        max_coordinates = Position(
            x=max(box.position.x + box.dimensions.x for box in boxes),
            y=max(box.position.y + box.dimensions.y for box in boxes),
        )
        return cls(
            position=position,
            dimensions=Dimensions(
                x=max_coordinates.x - position.x,
                y=max_coordinates.y - position.y,
            ),
            **kwargs,
        )



import glob
import os
from pathlib import Path
from typing import List, Tuple

from structurizr2csv.settings import EDGE_STYLE

ASSETS_PATH = Path(__file__).parent


class Style:
    """Build static styles for diagrams.net"""

    @classmethod
    def get(cls, style: str) -> str:
        with open(ASSETS_PATH / f"styles/{style}.txt", "r") as fp:
            return fp.read().strip()

    @classmethod
    @property
    def connector(cls) -> str:
        return cls.get("connector") + str(EDGE_STYLE)

    @classmethod
    def get_definitions(cls, path: str, ignored: Tuple[str, ...] = None) -> List[str]:
        """Returns all definitions for the styles contained in `path` directory."""
        defs = []
        if ignored is None:
            ignored = tuple()

        for file in sorted(glob.glob(str(ASSETS_PATH / path / "*.txt"))):
            key = os.path.basename(file)[:-4]
            if key not in ignored:
                with open(file, "r") as fp:
                    content = fp.read().strip().replace('"', "'")
                    defs.append(f'"{key}": "{content}"')

        return defs

    @classmethod
    def build_headers(cls) -> str:
        """Build all the styling headers for diagrams.net CSV format."""
        element_styles = cls.get_definitions(
            "styles", ignored=("styles", "connector", "group")
        )
        labels = cls.get_definitions("labels")

        with open(ASSETS_PATH / "styles/styles.txt", "r") as fp:
            styles = fp.read().strip()

        return styles.format(
            element_styles=",\\\n#  ".join(element_styles),
            labels=",\\\n#  ".join(labels),
        )



from typing import TYPE_CHECKING, Optional

from pydantic.fields import Field

from structurizr2csv.coordinates import Dimensions, Position
from structurizr2csv.elements.bounding_box import BoundingBox
from structurizr2csv.enums import Terminology
from structurizr2csv.settings import (
    PRIMARY_GROUP_BOTTOM_PADDING,
    PRIMARY_GROUP_PADDING,
    SECONDARY_GROUP_BOTTOM_PADDING,
    SECONDARY_GROUP_PADDING,
)
from structurizr2csv.style import Style
from structurizr2csv.utils import auto_index

if TYPE_CHECKING:
    from structurizr2csv.elements.box import Box  # noqa: F401


class Group(BoundingBox):
    """Convert a group/enterprise in structurizr to a parent box in diagrams.net"""

    id: str = Field(default_factory=lambda: auto_index("g", Group))

    name: str
    c4_type: Optional[Terminology]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.position = Position(
            x=self.position.x - self.padding,
            y=self.position.y - self.padding,
        )
        self.dimensions = Dimensions(
            x=self.dimensions.x + 2 * self.padding,
            y=self.dimensions.y + self.padding + self.bottom_padding,
        )

    @property
    def style(self) -> str:
        return "group_1" if self.is_primary else "group_2"

    @property
    def is_primary(self) -> bool:
        return self.c4_type is not None

    @property
    def padding(self) -> int:
        return PRIMARY_GROUP_PADDING if self.is_primary else SECONDARY_GROUP_PADDING

    @property
    def bottom_padding(self) -> int:
        return (
            PRIMARY_GROUP_BOTTOM_PADDING
            if self.is_primary
            else SECONDARY_GROUP_BOTTOM_PADDING
        )

    @property
    def csv_data(self):
        style = Style.get(self.style)

        return {
            "c4Name": self.name,
            "c4Type": self.c4_type.value if self.c4_type else "",
            "c4Description": "",
            "id": self.id,
            "parent": self.parent_id,
            "styleKey": f'"{style}"',
            "labelKey": self.style,
            "left": self.relative_position.x,
            "top": self.relative_position.y,
            "width": self.dimensions.x,
            "height": self.dimensions.y,
            "fill": "",
            "stroke": "",
        }



from typing import TYPE_CHECKING, Dict, Iterator, Optional, Tuple

from pydantic import BaseModel

from structurizr2csv.coordinates import Dimensions, Position

if TYPE_CHECKING:
    from structurizr2csv.elements.group import Group  # @UnusedImport


class BaseBox(BaseModel):
    """Base element for any diagrams.net object that is not a connector"""

    position: Position
    dimensions: Dimensions
    parent: Optional["Group"]

    # Attributes that can be edited in diagrams.net via "Edit Data..."
    _data_columns: Tuple[str, ...] = ("c4Name", "c4Type", "c4Description")

    # Other attributes that are not editable
    _meta_columns: Tuple[str, ...] = (
        "id",
        "parent",
        "styleKey",
        "labelKey",
        "left",
        "top",
        "width",
        "height",
        "fill",
        "stroke",
    )

    @classmethod
    def build_placeholders(cls, columns: Tuple[str]) -> str:
        return "{" + "},{".join(columns) + "}"

    @property
    def parent_id(self) -> str:
        return self.parent.id if self.parent else ""

    @property
    def style(self) -> str:
        """Returns the style for displaying the element in diagrams.net"""
        raise NotImplementedError

    @property
    def csv_data(self) -> Dict[str, str]:
        return {key: "" for key in self._data_columns + self._meta_columns}

    @property
    def relative_position(self) -> Position:
        """Returns the position within the element's parent group"""
        if self.parent:
            return Position(
                x=self.position.x - self.parent.position.x,
                y=self.position.y - self.parent.position.y,
            )
        else:
            return self.position

    def to_csv(self) -> str | Iterator[str]:
        return "{},{}".format(
            self.build_placeholders(self._data_columns),
            self.build_placeholders(self._meta_columns),
        ).format(**self.csv_data)



# generated by datamodel-codegen:
#   filename:  structurizr.yaml
#   timestamp: 2022-01-26T15:09:09+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class Enterprise(BaseModel):
    name: Optional[str] = Field(None, description="The name of the enterprise.")


class Location(Enum):
    External = "External"
    Internal = "Internal"
    Unspecified = "Unspecified"


class Location1(Enum):
    External = "External"
    Internal = "Internal"
    Unspecified = "Unspecified"


class Role(Enum):
    Primary = "Primary"
    Supporting = "Supporting"


class CodeElement(BaseModel):
    role: Optional[Role] = Field(None, description="The role of this code element.")
    name: Optional[str] = Field(None, description="The name of this code element.")
    type: Optional[str] = Field(
        None,
        description="The type of the code element (e.g. a fully qualified Java interface or class name).",
    )
    description: Optional[str] = Field(
        None, description="A short description of this component."
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    language: Optional[str] = Field(
        None,
        description='The programming language of this code element (e.g. "Java", "C#", etc).',
    )
    category: Optional[str] = Field(
        None, description="The category of code element; e.g. class, interface, etc."
    )
    visibility: Optional[str] = Field(
        None,
        description="The visibility of the code element; e.g. public, package, private.",
    )
    size: Optional[float] = Field(
        None, description="The size of the code element; e.g. the number of lines."
    )


class HttpHealthCheck(BaseModel):
    name: Optional[str] = Field(None, description="The name of the health check.")
    url: Optional[str] = Field(None, description="The health check URL/endpoint.")
    interval: Optional[float] = Field(
        None, description="The polling interval, in seconds."
    )
    timeout: Optional[float] = Field(
        None,
        description="The timeout after which a health check is deemed as failed, in milliseconds.",
    )
    headers: Optional[Dict[str, Any]] = Field(
        None,
        description="A set of name-value pairs corresponding to HTTP headers that should be sent with the request.",
    )


class InteractionStyle(Enum):
    Synchronous = "Synchronous"
    Asynchronous = "Asynchronous"


class Relationship(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this relationship in the model."
    )
    description: Optional[str] = Field(
        None, description="A short description of this relationship."
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this relationship.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this relationship can be found.",
    )
    sourceId: Optional[str] = Field(None, description="The ID of the source element.")
    destinationId: Optional[str] = Field(
        None, description="The ID of the destination element."
    )
    technology: Optional[str] = Field(
        None,
        description="The technology associated with this relationship (e.g. HTTPS, JDBC, etc).",
    )
    interactionStyle: Optional[InteractionStyle] = Field(
        None, description="The interaction style (synchronous or asynchronous)."
    )
    linkedRelationshipId: Optional[str] = Field(
        None,
        description="The ID of the container-container relationship upon which this container instance-container instance relationship is based.",
    )


class PaperSize(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class PaperSize1(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class PaperSize2(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class PaperSize3(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class PaperSize4(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class PaperSize5(Enum):
    A6_Portrait = "A6_Portrait"
    A6_Landscape = "A6_Landscape"
    A5_Portrait = "A5_Portrait"
    A5_Landscape = "A5_Landscape"
    A4_Portrait = "A4_Portrait"
    A4_Landscape = "A4_Landscape"
    A3_Portrait = "A3_Portrait"
    A3_Landscape = "A3_Landscape"
    A2_Portrait = "A2_Portrait"
    A2_Landscape = "A2_Landscape"
    A1_Portrait = "A1_Portrait"
    A1_Landscape = "A1_Landscape"
    A0_Portrait = "A0_Portrait"
    A0_Landscape = "A0_Landscape"
    Letter_Portrait = "Letter_Portrait"
    Letter_Landscape = "Letter_Landscape"
    Legal_Portrait = "Legal_Portrait"
    Legal_Landscape = "Legal_Landscape"
    Slide_4_3 = "Slide_4_3"
    Slide_16_9 = "Slide_16_9"
    Slide_16_10 = "Slide_16_10"


class Mode(Enum):
    Include = "Include"
    Exclude = "Exclude"


class FilteredView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    baseViewKey: Optional[str] = Field(
        None, description="The key of the view on which this filtered view is based."
    )
    mode: Optional[Mode] = Field(
        None,
        description="Whether elements/relationships are being included or excluded based upon the set of tags.",
    )
    tags: Optional[List[str]] = Field(
        None,
        description="The set of tags to include/exclude elements/relationships when rendering this filtered view.",
    )


class ElementView(BaseModel):
    id: Optional[str] = Field(None, description="The ID of the element.")
    x: Optional[float] = Field(
        None, description="The horizontal position of the element when rendered."
    )
    y: Optional[float] = Field(
        None, description="The vertical position of the element when rendered."
    )


class Routing(Enum):
    Direct = "Direct"
    Curved = "Curved"
    Orthogonal = "Orthogonal"


class Vertex(BaseModel):
    x: Optional[float] = Field(
        None, description="The horizontal position of the vertex when rendered."
    )
    y: Optional[float] = Field(
        None, description="The vertical position of the vertex when rendered."
    )


class AnimationStep(BaseModel):
    order: Optional[float] = Field(
        None, description="The order of this animation step."
    )
    elements: Optional[List[str]] = Field(
        None,
        description="The set of element IDs that should be included in this animation step.",
    )
    relationships: Optional[List[str]] = Field(
        None,
        description="The set of relationship IDs that should be included in this animation step.",
    )


class Dimensions(BaseModel):
    width: Optional[float] = Field(None, description="The width (pixels).")
    height: Optional[float] = Field(None, description="The height (pixels).")


class RankDirection(Enum):
    TopBottom = "TopBottom"
    BottomTop = "BottomTop"
    LeftRight = "LeftRight"
    RightLeft = "RightLeft"


class AutomaticLayout(BaseModel):
    rankDirection: Optional[RankDirection] = Field(
        None, description="The algorithm rank direction."
    )
    rankSeparation: Optional[float] = Field(
        None, description="The separation between ranks (pixels)."
    )
    nodeSeparation: Optional[float] = Field(
        None, description="The separation between nodes in the same rank (pixels)."
    )
    edgeSeparation: Optional[float] = Field(
        None, description="The separation between edges (pixels)."
    )
    vertices: Optional[bool] = Field(
        None, description="Whether vertices should be created during automatic layout."
    )


class MetadataSymbols(Enum):
    SquareBrackets_ = "SquareBrackets,"
    RoundBrackets_ = "RoundBrackets,"
    CurlyBrackets_ = "CurlyBrackets,"
    AngleBrackets_ = "AngleBrackets,"
    DoubleAngleBrackets_ = "DoubleAngleBrackets,"
    None_ = "None"


class Font(BaseModel):
    name: Optional[str] = Field(
        None, description='The font name (e.g. "Times New Roman", "Open Sans", etc).'
    )
    url: Optional[str] = Field(
        None, description="For web fonts, the URL where the font can be found."
    )


class Branding(BaseModel):
    logo: Optional[str] = Field(
        None, description="A Base64 data URI representation of a PNG/JPG/GIF file."
    )
    font: Optional[Font] = Field(
        None,
        description="Represents a font, including a name and an optional URL for web fonts.",
    )


class Shape(Enum):
    Box = "Box"
    RoundedBox = "RoundedBox"
    Component = "Component"
    Circle = "Circle"
    Ellipse = "Ellipse"
    Hexagon = "Hexagon"
    Diamond = "Diamond"
    Folder = "Folder"
    Cylinder = "Cylinder"
    Pipe = "Pipe"
    WebBrowser = "WebBrowser"
    MobileDevicePortrait = "MobileDevicePortrait"
    MobileDeviceLandscape = "MobileDeviceLandscape"
    Person = "Person"
    Robot = "Robot"


class Border(Enum):
    Solid = "Solid"
    Dashed = "Dashed"
    Dotted = "Dotted"


class ElementStyle(BaseModel):
    tag: Optional[str] = Field(
        None, description="The tag to which this element style applies."
    )
    width: Optional[float] = Field(
        None, description="The width of the element, in pixels."
    )
    height: Optional[float] = Field(
        None, description="The height of the element, in pixels."
    )
    background: Optional[str] = Field(
        None,
        description="The background colour of the element, as a HTML RGB hex string (e.g. '#ffffff').",
    )
    stroke: Optional[str] = Field(
        None,
        description="The stroke colour of the element, as a HTML RGB hex string (e.g. '#000000').",
    )
    color: Optional[str] = Field(
        None,
        description="The foreground (text) colour of the element, as a HTML RGB hex string (e.g. '#ffffff').",
    )
    fontSize: Optional[float] = Field(
        None, description="The standard font size used to render text, in pixels."
    )
    shape: Optional[Shape] = Field(
        None, description="The shape used to render the element."
    )
    icon: Optional[str] = Field(
        None, description="A Base64 data URI representation of a PNG/JPG/GIF file."
    )
    border: Optional[Border] = Field(
        None, description="The type of border used to render the element."
    )
    opacity: Optional[float] = Field(
        None, description="The opacity used when rendering the element; 0-100."
    )
    metadata: Optional[bool] = Field(
        None, description="Whether the element metadata should be shown or not."
    )
    description: Optional[bool] = Field(
        None, description="Whether the element description should be shown or not."
    )


class Routing1(Enum):
    Direct = "Direct"
    Curved = "Curved"
    Orthogonal = "Orthogonal"


class RelationshipStyle(BaseModel):
    tag: Optional[str] = Field(
        None, description="The tag to which this relationship style applies."
    )
    thickness: Optional[float] = Field(
        None, description="The thickness of the line, in pixels."
    )
    color: Optional[str] = Field(
        None,
        description="The colour of the line, as a HTML RGB hex string (e.g. '#ffffff').",
    )
    fontSize: Optional[float] = Field(
        None,
        description="The standard font size used to render the relationship annotation, in pixels.",
    )
    width: Optional[float] = Field(
        None, description="The width of the relationship annotation, in pixels."
    )
    dashed: Optional[bool] = Field(
        None,
        description="A flag to indicate whether the line is rendered as dashed or not.",
    )
    routing: Optional[Routing1] = Field(
        None, description="The routing algorithm used when rendering lines."
    )
    position: Optional[float] = Field(
        None,
        description="The position of the annotation along the line; 0 (start) to 100 (end).",
    )
    opacity: Optional[float] = Field(
        None, description="The opacity used when rendering the line; 0-100."
    )


class Format(Enum):
    Markdown = "Markdown"
    AsciiDoc = "AsciiDoc"


class DocumentationSection(BaseModel):
    title: Optional[str] = Field(
        None, description="The title (name/section heading) of the section."
    )
    content: Optional[str] = Field(
        None, description="The Markdown or AsciiDoc content of the section."
    )
    format: Optional[Format] = Field(None, description="The content format type.")
    order: Optional[float] = Field(
        None, description="The order (index) of the section in the document."
    )
    elementId: Optional[str] = Field(
        None,
        description="The ID of the element (in the model) that this section applies to (optional).",
    )


class Status(Enum):
    Proposed = "Proposed"
    Accepted = "Accepted"
    Superseded = "Superseded"
    Deprecated = "Deprecated"
    Rejected = "Rejected"


class Format1(Enum):
    Markdown = "Markdown"
    AsciiDoc = "AsciiDoc"


class Decision(BaseModel):
    id: Optional[str] = Field(None, description="The ID of the decision.")
    date: Optional[str] = Field(
        None, description="The date that the decision was made (ISO 8601 format)."
    )
    status: Optional[Status] = Field(None, description="The status of the decision.")
    title: Optional[str] = Field(None, description="The title of the decision.")
    content: Optional[str] = Field(
        None, description="The Markdown or AsciiDoc content of the section."
    )
    format: Optional[Format1] = Field(None, description="The content format type.")
    elementId: Optional[str] = Field(
        None,
        description="The ID of the element (in the model) that this decision applies to (optional).",
    )


class Image(BaseModel):
    name: Optional[str] = Field(None, description="The name of the image.")
    content: Optional[str] = Field(
        None, description="The (base64 encoded) content of the image."
    )
    type: Optional[str] = Field(
        None, description='The image MIME type (e.g. "image/png").'
    )


class DocumentationTemplateMetadata(BaseModel):
    name: Optional[str] = Field(
        None, description="The name of the documentation template."
    )
    author: Optional[str] = Field(
        None, description="The name of the author of the documentation template."
    )
    url: Optional[str] = Field(
        None,
        description="A URL that points to more information about the documentation template.",
    )


class Terminology(BaseModel):
    enterprise: Optional[str] = Field(
        None, description="The terminology used when rendering the enterprise boundary."
    )
    person: Optional[str] = Field(
        None, description="The terminology used when rendering people."
    )
    softwareSystem: Optional[str] = Field(
        None, description="The terminology used when rendering software systems."
    )
    container: Optional[str] = Field(
        None, description="The terminology used when rendering containers."
    )
    component: Optional[str] = Field(
        None, description="The terminology used when rendering components."
    )
    code: Optional[str] = Field(
        None, description="The terminology used when rendering code elements."
    )
    deploymentNode: Optional[str] = Field(
        None, description="The terminology used when rendering deployment nodes."
    )
    relationship: Optional[str] = Field(
        None, description="The terminology used when rendering relationships."
    )


class Role1(Enum):
    ReadWrite = "ReadWrite"
    ReadOnly = "ReadOnly"


class User(BaseModel):
    username: Optional[str] = Field(
        None, description="The username of the user (e.g. e-mail address)."
    )
    role: Optional[Role1] = Field(None, description="The user's role.")


class APIResponse(BaseModel):
    success: Optional[bool] = Field(
        None, description="true if the API call was successful, false otherwise."
    )
    message: Optional[str] = Field(
        None, description="A human readable response message."
    )
    revision: Optional[int] = Field(None, description="The internal revision number.")


class Person(BaseModel):
    id: Optional[str] = Field(None, description="The ID of this person in the model.")
    name: Optional[str] = Field(None, description="The name of this person.")
    description: Optional[str] = Field(
        None, description="A short description of this person."
    )
    tags: Optional[str] = Field(
        None, description="A comma separated list of tags associated with this person."
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    location: Optional[Location] = Field(
        None, description="The location of this person."
    )
    group: Optional[str] = Field(
        None,
        description="The name of the group in which this person should be included in.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None, description="The set of relationships from this person to other elements."
    )


class Component(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this component in the model."
    )
    name: Optional[str] = Field(None, description="The name of this component.")
    description: Optional[str] = Field(
        None, description="A short description of this component."
    )
    technology: Optional[str] = Field(
        None,
        description="The technology associated with this component (e.g. Spring Bean).",
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this component.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    code: Optional[List[CodeElement]] = Field(
        None, description="The set of code elements that make up this component."
    )
    group: Optional[str] = Field(
        None,
        description="The name of the group in which this component should be included in.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this component to other elements.",
    )


class InfrastructureNode(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this infrastructure node in the model."
    )
    name: Optional[str] = Field(
        None, description="The name of this infrastructure node."
    )
    description: Optional[str] = Field(
        None, description="A short description of this infrastructure node."
    )
    technology: Optional[str] = Field(
        None,
        description='The technology associated with this infrastructure node (e.g. "Route 53").',
    )
    environment: Optional[str] = Field(
        None,
        description='The deployment environment in which this infrastructure node resides (e.g. "Development", "Live", etc).',
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this infrastructure node.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this infrastructure node to other elements.",
    )


class SoftwareSystemInstance(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this software system instance in the model."
    )
    softwareSystemId: Optional[str] = Field(
        None, description="The ID of the software system this is an instance of."
    )
    instanceId: Optional[float] = Field(
        None, description="The number/index of this instance."
    )
    environment: Optional[str] = Field(
        None,
        description='The deployment environment in which this software system instance resides (e.g. "Development", "Live", etc).',
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this software system instance.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this software system instance to other elements.",
    )
    healthChecks: Optional[List[HttpHealthCheck]] = Field(
        None,
        description="The set of HTTP-based health checks for this software system instance.",
    )


class ContainerInstance(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this container instance in the model."
    )
    containerId: Optional[str] = Field(
        None, description="The ID of the container this is an instance of."
    )
    instanceId: Optional[float] = Field(
        None, description="The number/index of this instance."
    )
    environment: Optional[str] = Field(
        None,
        description='The deployment environment in which this container instance resides (e.g. "Development", "Live", etc).',
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this container instance.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this container instance to other elements.",
    )
    healthChecks: Optional[List[HttpHealthCheck]] = Field(
        None,
        description="The set of HTTP-based health checks for this container instance.",
    )


class RelationshipView(BaseModel):
    id: Optional[str] = Field(None, description="The ID of the relationship.")
    description: Optional[str] = Field(
        None,
        description="The description of this relationship (used in dynamic views only).",
    )
    response: Optional[bool] = Field(
        None,
        description="Signifies whether this relationship represents a return/response message (used in dynamic views only).",
    )
    order: Optional[str] = Field(
        None,
        description="Gets the order of this relationship (used in dynamic views only; e.g. 1.0, 1.1, 2.0, etc).",
    )
    vertices: Optional[List[Vertex]] = Field(
        None, description="The set of vertices used to render the relationship."
    )
    routing: Optional[Routing] = Field(
        None,
        description="The routing algorithm used when rendering this individual relationship.",
    )
    position: Optional[float] = Field(
        None,
        description="The position of the annotation along the line; 0 (start) to 100 (end).",
    )


class Styles(BaseModel):
    elements: Optional[List[ElementStyle]] = Field(
        None, description="The set of element styles."
    )
    relationships: Optional[List[RelationshipStyle]] = Field(
        None, description="The set of relationship styles."
    )


class Configuration(BaseModel):
    styles: Optional[Styles] = Field(
        None, description="The styles associated with this set of views."
    )
    lastSavedView: Optional[str] = Field(
        None, description="The key of the view that was saved most recently."
    )
    defaultView: Optional[str] = Field(
        None, description="The key of the view that should be shown by default."
    )
    themes: Optional[List[str]] = Field(
        None,
        description="The URL(s) of the theme(s) to be used when rendering diagrams.",
    )
    branding: Optional[Branding] = None
    terminology: Optional[Terminology] = None
    metadataSymbols: Optional[MetadataSymbols] = Field(
        None, description="The type of symbols to use when rendering metadata."
    )


class Documentation(BaseModel):
    sections: Optional[List[DocumentationSection]] = None
    decisions: Optional[List[Decision]] = None
    images: Optional[List[Image]] = None
    template: Optional[DocumentationTemplateMetadata] = None


class WorkspaceConfiguration(BaseModel):
    users: Optional[List[User]] = None


class Container(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this container in the model."
    )
    name: Optional[str] = Field(None, description="The name of this container.")
    description: Optional[str] = Field(
        None, description="A short description of this container."
    )
    technology: Optional[str] = Field(
        None,
        description="The technology associated with this container (e.g. Apache Tomcat).",
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this container.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    components: Optional[List[Component]] = Field(
        None, description="The set of components within this container."
    )
    group: Optional[str] = Field(
        None,
        description="The name of the group in which this container should be included in.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this container to other elements.",
    )


class DeploymentNode(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this deployment node in the model."
    )
    name: Optional[str] = Field(None, description="The name of this deployment node.")
    description: Optional[str] = Field(
        None, description="A short description of this deployment node."
    )
    technology: Optional[str] = Field(
        None,
        description="The technology associated with this deployment node (e.g. Apache Tomcat).",
    )
    environment: Optional[str] = Field(
        None,
        description='The deployment environment in which this deployment node resides (e.g. "Development", "Live", etc).',
    )
    instances: Optional[float] = Field(None, description="The number of instances.")
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this deployment node.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    children: Optional[List[DeploymentNode]] = Field(
        None, description="The set of child/nested deployment nodes."
    )
    infrastructureNodes: Optional[List[InfrastructureNode]] = None
    softwareSystemInstances: Optional[List[SoftwareSystemInstance]] = Field(
        None,
        description="The set of software systems instances running in this deployment node..",
    )
    containerInstances: Optional[List[ContainerInstance]] = Field(
        None,
        description="The set of container instances running in this deployment node..",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this deployment node to other elements.",
    )


class SystemLandscapeView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    paperSize: Optional[PaperSize] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    enterpriseBoundaryVisible: Optional[bool] = Field(
        None,
        description="Specifies whether the enterprise boundary (to differentiate internal elements from external elements) should be visible on the resulting diagram.",
    )
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this views."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this views."
    )
    animations: Optional[List[AnimationStep]] = Field(
        None, description="The set of animation steps (optional)."
    )


class SystemContextView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    softwareSystemId: Optional[str] = Field(
        None, description="The ID of the software system this view is associated with."
    )
    paperSize: Optional[PaperSize1] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    enterpriseBoundaryVisible: Optional[bool] = Field(
        None,
        description='Specifies whether the enterprise boundary (to differentiate internal elements from external elements") should be visible on the resulting diagram.',
    )
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this view."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this view."
    )
    animations: Optional[List[AnimationStep]] = Field(
        None, description="The set of animation steps (optional)."
    )


class ContainerView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    softwareSystemId: Optional[str] = Field(
        None, description="The ID of the software system this view is associated with."
    )
    paperSize: Optional[PaperSize2] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this views."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this views."
    )
    animations: Optional[List[AnimationStep]] = Field(
        None, description="The set of animation steps (optional)."
    )
    externalSoftwareSystemBoundariesVisible: Optional[bool] = Field(
        None,
        description='Specifies whether software system boundaries should be visible for "external" containers (those outside the software system in scope).',
    )


class ComponentView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    containerId: Optional[str] = Field(
        None, description="The ID of the container this view is associated with."
    )
    paperSize: Optional[PaperSize3] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this views."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this views."
    )
    animations: Optional[List[AnimationStep]] = Field(
        None, description="The set of animation steps (optional)."
    )
    externalContainerBoundariesVisible: Optional[bool] = Field(
        None,
        description='Specifies whether container boundaries should be visible for "external" components (those outside the container in scope).',
    )


class DynamicView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="An identifier for this view.")
    elementId: Optional[str] = Field(
        None,
        description="The ID of the element this view is associated with (optional).",
    )
    paperSize: Optional[PaperSize4] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this views."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this views."
    )
    externalBoundariesVisible: Optional[bool] = Field(
        None,
        description='Specifies software system/container boundaries should be visible for "external" containers/components (those outside the element in scope)',
    )


class DeploymentView(BaseModel):
    title: Optional[str] = Field(None, description="The title of this view (optional).")
    description: Optional[str] = Field(
        None, description="The description of this view."
    )
    key: Optional[str] = Field(None, description="A unique identifier for this view.")
    softwareSystemId: Optional[str] = Field(
        None,
        description="The ID of the software system this view is associated with (optional).",
    )
    environment: Optional[str] = Field(
        None,
        description='The name of the environment that this deployment view is for (e.g. "Development", "Live", etc).',
    )
    paperSize: Optional[PaperSize5] = Field(
        None, description="The paper size that should be used to render this view."
    )
    dimensions: Optional[Dimensions] = None
    automaticLayout: Optional[AutomaticLayout] = None
    elements: Optional[List[ElementView]] = Field(
        None, description="The set of elements in this views."
    )
    relationships: Optional[List[RelationshipView]] = Field(
        None, description="The set of relationships in this views."
    )
    animations: Optional[List[AnimationStep]] = Field(
        None, description="The set of animation steps (optional)."
    )


class SoftwareSystem(BaseModel):
    id: Optional[str] = Field(
        None, description="The ID of this software system in the model."
    )
    name: Optional[str] = Field(None, description="The name of this software system.")
    description: Optional[str] = Field(
        None, description="A short description of this software system."
    )
    location: Optional[Location1] = Field(
        None, description="The location of this software system."
    )
    tags: Optional[str] = Field(
        None,
        description="A comma separated list of tags associated with this software system.",
    )
    url: Optional[str] = Field(
        None,
        description="The URL where more information about this element can be found.",
    )
    containers: Optional[List[Container]] = Field(
        None, description="The set of containers within this software system."
    )
    group: Optional[str] = Field(
        None,
        description="The name of the group in which this software system should be included in.",
    )
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )
    relationships: Optional[List[Relationship]] = Field(
        None,
        description="The set of relationships from this software system to other elements.",
    )


class Views(BaseModel):
    systemLandscapeViews: Optional[List[SystemLandscapeView]] = Field(
        None, description="The set of system landscape views."
    )
    systemContextViews: Optional[List[SystemContextView]] = Field(
        None, description="The set of system context views."
    )
    containerViews: Optional[List[ContainerView]] = Field(
        None, description="The set of container views."
    )
    componentViews: Optional[List[ComponentView]] = Field(
        None, description="The set of component views."
    )
    dynamicViews: Optional[List[DynamicView]] = Field(
        None, description="The set of dynamic views."
    )
    deploymentView: Optional[List[DeploymentView]] = Field(
        None, description="The set of deployment views."
    )
    filteredViews: Optional[List[FilteredView]] = Field(
        None, description="The set of filtered views."
    )
    configuration: Optional[Configuration] = None


class Model(BaseModel):
    enterprise: Optional[Enterprise] = Field(
        None, description="The enterprise associated with this model."
    )
    people: Optional[List[Person]] = None
    softwareSystems: Optional[List[SoftwareSystem]] = None
    deploymentNodes: Optional[List[DeploymentNode]] = None


class Workspace(BaseModel):
    id: Optional[float] = Field(None, description="The workspace ID.")
    name: Optional[str] = Field(None, description="The name of the workspace.")
    description: Optional[str] = Field(
        None, description="A short description of the workspace."
    )
    version: Optional[str] = Field(
        None, description="A version number for the workspace."
    )
    thumbnail: Optional[str] = Field(
        None,
        description="The thumbnail associated with the workspace; a Base64 encoded PNG file as a data URI (data:image/png;base64).",
    )
    lastModifiedDate: Optional[str] = Field(
        None,
        description='The last modified date, in ISO 8601 format (e.g. "2018-09-08T12:40:03Z").',
    )
    lastModifiedUser: Optional[str] = Field(
        None,
        description="A string identifying the user who last modified the workspace (e.g. an e-mail address or username).",
    )
    lastModifiedAgent: Optional[str] = Field(
        None,
        description='A string identifying the agent that was last used to modify the workspace (e.g. "structurizr-java/1.2.0").',
    )
    model: Optional[Model] = None
    views: Optional[Views] = None
    documentation: Optional[Documentation] = None
    configuration: Optional[WorkspaceConfiguration] = None
    properties: Optional[Dict[str, Any]] = Field(
        None, description="A set of arbitrary name-value properties."
    )


DeploymentNode.update_forward_refs()



from structurizr2csv.enums import EdgeStyle

DOCKER_MOUNT_TARGET = "/structurizr2csv"

# This must match the workdir of the official structurizr/cli image
DOCKER_STRUCTURIZR_MOUNT_TARGET = "/usr/local/structurizr"

EDGE_STYLE = EdgeStyle.DIRECT

PRIMARY_GROUP_PADDING = 20
PRIMARY_GROUP_BOTTOM_PADDING = 60

SECONDARY_GROUP_PADDING = 10
SECONDARY_GROUP_BOTTOM_PADDING = 40

LEGEND_OFFSET_FROM_TITLE = 20

# Arbitrary sizing factor to adapt the positions so that the result layout looks OK.
# Structurizr diagrams are sized for 300 dpi, diagrams.net a priori for 100 dpi, but
# 0.33 will give bad results because the boxes' dimensions are not the same.
POSITION_FACTOR = 0.4

FILL_COLORS = {
    "person": "#083F75",
    "person-external": "#6C6477",
    "softwareSystem": "#1061B0",
    "softwareSystem-external": "#8C8496",
    "softwareSystem-existing": "#8C8496",
    "container": "#23A2D9",
    "container-external": "#8C8496",
    "component": "#63BEF2",
    "component-external": "#8C8496",
}
STROKE_COLORS = {
    "person": "#06315C",
    "person-external": "#4D4D4D",
    "softwareSystem": "#0D5091",
    "softwareSystem-external": "#736782",
    "softwareSystem-existing": "#736782",
    "container": "#0E7DAD",
    "container-external": "#8C8496",
    "component": "#2086C9",
    "component-external": "#8C8496",
}



import shutil
from itertools import count
from pathlib import Path
from typing import Type

from pydantic import BaseModel


def camel_case(name: str) -> str:
    return "".join([name[0].lower(), name[1:]])


class Directory:
    @classmethod
    def create(cls, path: Path):
        print(f"Creating {path}")
        path.mkdir(parents=True, exist_ok=False)

    @classmethod
    def clean(cls, path: Path):
        print(f"Cleaning {path}")
        for child in path.iterdir():
            try:
                child.unlink()
            except IsADirectoryError:
                cls.remove(child)

    @classmethod
    def remove(cls, path: Path):
        print(f"Removing {path}")
        shutil.rmtree(path)  # also works when not empty

    @classmethod
    def create_or_clean(cls, path: Path):
        if path.exists():
            cls.clean(path)
        else:
            cls.create(path)


def auto_index(prefix: str, parent_cls: Type[BaseModel]) -> str:
    """Generate an auto-incremented string index for pydantic objects."""
    try:
        parent_cls._counter  # type:ignore
    except AttributeError:

        @classmethod
        def reset_counter(cls):
            cls._counter = count(start=1)

        # I somehow prefer this dirty stuff, rather than copy/pasting to
        # define reset_counter() directly in the classes' definitions...
        parent_cls.reset_counter = reset_counter  # type:ignore
        parent_cls.reset_counter()  # type:ignore

    return f"{prefix}{parent_cls._counter.__next__()}"  # type:ignore



from enum import Enum


class ParameterEnum(Enum):
    def __str__(self) -> str:
        return (
            f"{self.config.param}={self.value}" if self.value else ""
        )  # pyright: reportGeneralTypeIssues=false


class EdgeStyle(ParameterEnum):
    class Config:
        param = "edgeStyle"

    # this is not defined in jgraph API, but there are apparently no constant to
    # display the edge as a direct segment between the two linked elements
    # (even "segmentEdgeStyle" is not doing that).
    DIRECT = None

    # Redefine for our purpose some constants from
    # https://jgraph.github.io/mxgraph/docs/js-api/files/util/mxConstants-js.html
    ELBOW = "elbowEdgeStyle"
    ENTITY_RELATION = "entityRelationEdgeStyle"
    LOOP = "loopEdgeStyle"
    SIDE_TO_SIDE = "sideToSideEdgeStyle"
    TOP_TO_BOTTOM = "topToBottomEdgeStyle"
    ORTHOGONAL = "orthogonalEdgeStyle"
    SEGMENT = "segmentEdgeStyle"


class Tag(Enum):
    # tag that we use to display boxes in grey
    EXISTING_SYSTEM = "Existing System"

    # tags for which a dedicated style is defined
    HEXAGON = "Hexagon"
    CYLINDER = "Cylinder"
    PIPE = "Pipe"
    WEB_BROWSER = "Web Browser"

    # tag aliases
    DATABASE = "Database"
    MOBILE_APP = "Mobile App"


class Terminology(Enum):
    legend = "Legend"
    person = "Person"
    persons = "Persons"
    enterprise = "Enterprise"
    systemLandscape = "System Landscape"
    systemContext = "System Context"
    softwareSystem = "Software System"
    container = "Container"
    containers = "Containers"
    component = "Component"
    components = "Components"
    dynamic = "Dynamic"
    relationship = "Relationship"



import json
import os
import socket
import textwrap
import warnings
from pathlib import Path
from typing import Optional

from structurizr2csv.settings import (
    DOCKER_MOUNT_TARGET,
    DOCKER_STRUCTURIZR_MOUNT_TARGET,
)

# c.f. https://github.com/docker/docker-py/issues/2928
with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    import docker


class DSL2JSON:
    """Convert structurizr DSL to JSON and apply GraphViz autolayout."""

    def __init__(self, input_path: Path):
        self.input_path = input_path

    @property
    def output_path(self) -> Path:
        return self.input_path.with_suffix(".json")  # i.e. *.json

    @property
    def tmp_dsl_path(self) -> Path:
        return self.input_path.with_suffix(".autolayout.dsl")  # i.e. *.autolayout.dsl

    @property
    def tmp_output_path(self) -> Path:
        return self.tmp_dsl_path.with_suffix(".json")  # i.e. *.autolayout.json

    @property
    def autolayout_dsl_content(self):
        """Extend the workspace to add GraphViz autolayout."""
        # c.f. https://github.com/structurizr/cli/issues/62#issuecomment-999623728
        return textwrap.dedent(
            f"""
workspace extends {self.input_path.name} {{
    !script groovy {{
        new com.structurizr.graphviz.GraphvizAutomaticLayout().apply(workspace);
    }}
}}"""
        )

    def clean_temporary_artifacts(self):
        """Remove all temporary files that have been created during the conversion."""
        if self.tmp_output_path.exists():
            with open(self.tmp_output_path, "r") as fp:
                for key, values in json.load(fp)["views"].items():
                    if key != "configuration":
                        for view in values:
                            view_key = view["key"]
                            for suffix in (".dot", ".dot.svg"):
                                (self.input_path.parent / f"{view_key}{suffix}").unlink(
                                    missing_ok=True
                                )

        self.tmp_dsl_path.unlink(missing_ok=True)

    def get_docker_host_working_directory(
        self, client: docker.client.DockerClient
    ) -> Optional[str]:
        """Returns the host's working directory if we are running in docker, else None.

        If we are currently running in docker, the host's working directory must be the
        source path of the mounted /structurizr2csv volume.
        """
        # In Makefile, we fake "docker in docker" (dind) using the socket solution.
        # So here, we assume to be inside a docker container and we will run another
        # container using the existing docker server that is running on the host.
        # So the mount source path that we will pass to structurizr's docker must
        # be the host's source path, and not the target in structurizr2csv's container.
        # c.f. https://jpetazzo.github.io/2015/09/03/do-not-use-docker-in-docker-for-ci/
        try:
            container_info = client.api.inspect_container(socket.gethostname())
        except docker.errors.NotFound:  # pyright: reportGeneralTypeIssues=false
            # we are running in docker
            return None
        else:
            path = [
                mount
                for mount in container_info["Mounts"]
                if mount["Destination"] == DOCKER_MOUNT_TARGET
            ][0]["Source"]

            print(f"The mounted volume host path is {path}")  # e.g. /d/structurizr2csv
            return path

    def get_docker_host_workspace_directory(
        self, client: docker.client.DockerClient
    ) -> str:
        """Returns the host's workspace directory (that contains the DSL/JSON files)."""
        new_volume_source_path = os.path.realpath(self.input_path.parent)

        mounted_volume_host_path = self.get_docker_host_working_directory(client)
        if mounted_volume_host_path:  # we are currently in docker
            new_volume_source_path = new_volume_source_path.replace(
                DOCKER_MOUNT_TARGET, mounted_volume_host_path
            )

        # e.g. /d/structurizr2csv/examples
        print(f"The new volume source path will be {new_volume_source_path}")
        return new_volume_source_path

    def convert(self):
        with open(self.tmp_dsl_path, "w") as fp:
            fp.write(self.autolayout_dsl_content)

        # c.f. https://github.com/docker/docker-py/issues/2928
        # TODO: remove this catch when docker package has been updated
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", category=DeprecationWarning)

            client = docker.from_env()
            try:
                client.containers.run(
                    image="souliane/structurizr-cli-with-graphviz",
                    command=(f'export -w "{self.tmp_dsl_path.name}" -f json'),
                    volumes={
                        self.get_docker_host_workspace_directory(client): {
                            "bind": DOCKER_STRUCTURIZR_MOUNT_TARGET,
                            "mode": "rw",
                        }
                    },
                    auto_remove=True,
                )
            finally:
                self.clean_temporary_artifacts()
                client.close()

        self.output_path.unlink(missing_ok=True)
        self.tmp_output_path.rename(self.output_path)

        return self.output_path



from collections import defaultdict
from typing import Dict, Optional, Set

from pydantic import BaseModel, Field

from structurizr2csv.style import Style
from structurizr2csv.utils import auto_index
from structurizr2csv.workspace_extension import Element


class Connector(BaseModel):
    """Convert a relationship in structurizr to a connector in diagrams.net"""

    id: str = Field(default_factory=lambda: auto_index("c", Connector))

    description: str
    technology: Optional[str]

    # Map structurizr destination element to the source elements
    instances: Dict[Element, Set[Element]] = Field(
        default_factory=lambda: defaultdict(set)
    )

    @property
    def label(self):
        return (
            f"{self.description}<br>[{self.technology}]"
            if self.technology
            else self.description
        )

    def __str__(self):
        return (
            f'# connect: {{"from": "{self.id}","to": "id",'
            f'"invert": "true","label": "{self.label}",'
            f'"style": "{Style.connector}"}}\n'
        )
